﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;

namespace TestLibrary
{
    /// <summary>
    /// 메뉴 버튼
    /// </summary>
    public partial class MenuButton 
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region UI 테마 속성 - UIThemeProperty

        /// <summary>
        /// UI 테마 속성
        /// </summary>
        public static readonly DependencyProperty UIThemeProperty = DependencyProperty.Register
        (
            "UITheme",
            typeof(UITheme),
            typeof(MenuButton)
        );

        #endregion
        #region HOVER 배경 속성 - HoverBackgroundProperty

        /// <summary>
        /// HOVER 배경 속성
        /// </summary>
        public static readonly DependencyProperty HoverBackgroundProperty = DependencyProperty.Register
        (
            "HoverBackground",
            typeof(SolidColorBrush),
            typeof(MenuButton)
        );

        #endregion
        #region 애니메이션 속도 속성 - AnimationSpeedProperty

        /// <summary>
        /// 애니메이션 속도 속성
        /// </summary>
        public static readonly DependencyProperty AnimationSpeedProperty = DependencyProperty.Register
        (
            "AnimationSpeed",
            typeof(TimeSpan),
            typeof(MenuButton)
        );

        #endregion
        #region 이미지 속성 - ImageProperty

        /// <summary>
        /// 이미지 속성
        /// </summary>
        public static readonly DependencyProperty ImageProperty = DependencyProperty.Register
        (
            "Image",
            typeof(ImageSource),
            typeof(MenuButton)
        );

        #endregion
        #region 텍스트 속성 - TextProperty

        /// <summary>
        /// 텍스트 속성
        /// </summary>
        public static readonly DependencyProperty TextProperty = DependencyProperty.Register
        (
            "Text",
            typeof(string),
            typeof(MenuButton)
        );

        #endregion
        #region 자식 리스트 속성 - ChildrenProperty

        /// <summary>
        /// 자식 리스트 속성
        /// </summary>
        public static readonly DependencyProperty ChildrenProperty = DependencyProperty.Register
        (
            "Children",
            typeof(List<MenuButton>),
            typeof(MenuButton)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 자식 표시 여부
        /// </summary>
        private bool childrenVisible;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 테마 - Theme

        /// <summary>
        /// 테마
        /// </summary>
        public UITheme Theme
        {
            get
            {
                return (UITheme)GetValue(UIThemeProperty);
            }
            set
            {
                SetValue(UIThemeProperty, value);

                BitmapImage bitmapImage;

                switch(Theme)
                {
                    case UITheme.Light :

                        bitmapImage = new BitmapImage(new Uri(@"IMAGE/Light24.png", UriKind.Relative));

                        break;

                    case UITheme.Dark :

                        bitmapImage = new BitmapImage(new Uri(@"IMAGE/Dark24.png", UriKind.Relative));

                        break;

                    default :

                        throw new ArgumentOutOfRangeException();
                }

                Image image = FindName("chevronImage") as Image;

                image.Source = bitmapImage;

                Label textLabel = FindName("textLabel") as Label;

                if(Theme == UITheme.Light)
                {
                    textLabel.Foreground = Brushes.WhiteSmoke;
                }
                else
                {
                    textLabel.Foreground = new SolidColorBrush { Color = Color.FromRgb(30, 30, 30) };
                }
            }
        }

        #endregion
        #region HOVER 배경 - HoverBackground

        /// <summary>
        /// HOVER 배경
        /// </summary>
        public SolidColorBrush HoverBackground
        {
            get
            {
                return (SolidColorBrush)GetValue(HoverBackgroundProperty);
            }
            set
            {
                SetValue(HoverBackgroundProperty, value);
            }
        }

        #endregion
        #region 애니메이션 속도 - AnimationSpeed

        /// <summary>
        /// 애니메이션 속도
        /// </summary>
        public TimeSpan AnimationSpeed
        {
            get
            {
                return (TimeSpan)GetValue(AnimationSpeedProperty);
            }
            set
            {
                SetValue(AnimationSpeedProperty, value);
            }
        }

        #endregion
        #region 이미지 - Image

        /// <summary>
        /// 이미지
        /// </summary>
        public ImageSource Image
        {
            get
            {
                return (ImageSource)GetValue(ImageProperty);
            }
            set
            {
                SetValue(ImageProperty, value);
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                return (string)GetValue(TextProperty);
            }
            set
            {
                SetValue(TextProperty, value);
            }
        }

        #endregion
        #region 자식 리스트 - Children

        /// <summary>
        /// 자식 리스트
        /// </summary>
        public List<MenuButton> Children
        {
            get
            {
                return (List<MenuButton>)GetValue(ChildrenProperty);
            }
            set
            {
                SetValue(ChildrenProperty, value);

                foreach(MenuButton child in Children)
                {
                    child.ParentButton = this;
                }

                DataContext = Children;

                Image chevronImage = FindName("chevronImage") as Image;

                if(Children == null || Children.Count == 0)
                {
                    chevronImage.Visibility = Visibility.Hidden;
                }
                else
                {
                    chevronImage.Visibility = Visibility.Visible;
                }
            }
        }

        #endregion

        #region 부모 버튼 - ParentButton

        /// <summary>
        /// 부모 버튼
        /// </summary>
        public MenuButton ParentButton { get; private set; }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 확장 높이 - ExpandedHeight

        /// <summary>
        /// 확장 높이
        /// </summary>
        private double ExpandedHeight => Children.Count * ActualHeight + 5;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MenuButton()

        /// <summary>
        /// 생성자
        /// </summary>
        public MenuButton()
        {
            InitializeComponent();

            Children = new List<MenuButton>();

            Theme = UITheme.Light;

            AnimationSpeed = TimeSpan.FromMilliseconds(150);

            MouseEnter                  += UserControl_MouseEnter;
            MouseLeave                  += UserControl_MouseLeave;
            this.triggerGrid.MouseDown  += UserControl_MouseDown;
            this.chevronImage.MouseDown += UserControl_MouseDown;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 템플리트 적용시 처리하기 - OnApplyTemplate()

        /// <summary>
        /// 템플리트 적용시 처리하기
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            Label textLabel = FindName("textLabel") as Label;

            textLabel.Content = Text;

            Image image = FindName("image") as Image;

            image.Source = Image;

            ItemsControl childItemsControl = FindName("childItemsControl") as ItemsControl;

            childItemsControl.DataContext = Children;

            childItemsControl.GetBindingExpression(ItemsControl.ItemsSourceProperty).UpdateTarget();

            foreach(MenuButton child in Children)
            {
                child.ParentButton = this;
            }

            Image chevronImage = FindName("chevronImage") as Image;

            if(Children == null || Children.Count == 0)
            {
                chevronImage.Visibility = Visibility.Hidden;
            }
            else
            {
                chevronImage.Visibility = Visibility.Visible;
            }

            Grid rootGrid = FindName("rootGrid") as Grid;

            rootGrid.Background = Background;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 초기화시 처리하기 - OnInitialized(e)

        /// <summary>
        /// 초기화시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnInitialized(EventArgs e)
        {
            InitializeComponent();

            base.OnInitialized(e);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 사용자 컨트롤 마우스 진입시 처리하기 - UserControl_MouseEnter(sender, e)

        /// <summary>
        /// 사용자 컨트롤 마우스 진입시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void UserControl_MouseEnter(object sender, MouseEventArgs e)
        {
            Grid rootGrid = FindName("rootGrid") as Grid;

            ColorAnimation animation = new ColorAnimation
            {
                To       = HoverBackground.Color,
                Duration = AnimationSpeed
            };

            Storyboard storyboard = new Storyboard();

            storyboard.Children.Add(animation);

            Storyboard.SetTarget        (animation, rootGrid);
            Storyboard.SetTargetProperty(animation, new PropertyPath("Background.Color"));

            storyboard.Begin();
        }

        #endregion
        #region 사용자 컨트롤 마우스 이탈시 처리하기 - UserControl_MouseLeave(sender, e)

        /// <summary>
        /// 사용자 컨트롤 마우스 이탈시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void UserControl_MouseLeave(object sender, MouseEventArgs e)
        {
            Grid rootGrid = FindName("rootGrid") as Grid;

            ColorAnimation animation = new ColorAnimation
            {
                To       = (Background as SolidColorBrush).Color,
                Duration = AnimationSpeed
            };

            Storyboard storyboard = new Storyboard();

            storyboard.Children.Add(animation);

            Storyboard.SetTarget        (animation, rootGrid);
            Storyboard.SetTargetProperty(animation, new PropertyPath("Background.Color"));

            storyboard.Begin();
        }

        #endregion
        #region 사용자 컨트롤 마우스 DOWN 처리하기 - UserControl_MouseDown(sender, e)

        /// <summary>
        /// 사용자 컨트롤 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void UserControl_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if(Children.Count == 0)
            {
                return;
            }

            if(this.childrenVisible)
            {
                MenuButton parent = ParentButton;

                while(parent != null)
                {
                    ItemsControl childItemsControl = FindName("childItemsControl") as ItemsControl;

                    parent.Reduce(childItemsControl.ActualHeight);

                    parent = parent.ParentButton;
                }

                Close();
            }
            else
            {
                MenuButton parent = ParentButton;

                while(parent != null)
                {
                    parent.Expand(ExpandedHeight);

                    parent = parent.ParentButton;
                }

                Open();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 확장하기 - Expand(height)

        /// <summary>
        /// 확장하기
        /// </summary>
        /// <param name="height">높이</param>
        private void Expand(double height)
        {
            ItemsControl childItemsControl = FindName("childItemsControl") as ItemsControl;

            DoubleAnimation animation = new DoubleAnimation
            {
                From     = childItemsControl.ActualHeight,
                To       = childItemsControl.ActualHeight + height,
                Duration = AnimationSpeed
            };

            childItemsControl.BeginAnimation(HeightProperty, animation);
        }

        #endregion
        #region 축소하기 - Reduce(height)

        /// <summary>
        /// 축소하기
        /// </summary>
        /// <param name="height">높이</param>
        private void Reduce(double height)
        {
            ItemsControl childItemsControl = FindName("childItemsControl") as ItemsControl;

            DoubleAnimation animation = new DoubleAnimation
            {
                From     = childItemsControl.ActualHeight,
                To       = childItemsControl.ActualHeight - height,
                Duration = AnimationSpeed
            };

            childItemsControl.BeginAnimation(HeightProperty, animation);
        }

        #endregion

        #region 열기 - Open()

        /// <summary>
        /// 열기
        /// </summary>
        private void Open()
        {
            RotateTransform chevronRotateTransform = FindName("chevronRotateTransform") as RotateTransform;

            DoubleAnimation animation = new DoubleAnimation
            {
                To       = 90,
                Duration = AnimationSpeed
            };

            chevronRotateTransform.BeginAnimation(RotateTransform.AngleProperty, animation);

            Expand(ExpandedHeight);

            this.childrenVisible = true;
        }

        #endregion
        #region 닫기 - Close()

        /// <summary>
        /// 닫기
        /// </summary>
        private void Close()
        {
            RotateTransform chevronRotateTransform = FindName("chevronRotateTransform") as RotateTransform;

            DoubleAnimation angleDoubleAnimation = new DoubleAnimation
            {
                To = 0,
                Duration = AnimationSpeed
            };

            chevronRotateTransform.BeginAnimation(RotateTransform.AngleProperty, angleDoubleAnimation);

            foreach(MenuButton child in Children)
            {
                child.Close();
            }

            if(!this.childrenVisible)
            {
                return;
            }

            ItemsControl childItemsControl = FindName("childItemsControl") as ItemsControl;

            DoubleAnimation heightDoubleAnimation = new DoubleAnimation
            {
                From     = childItemsControl.ActualHeight,
                To       = 0,
                Duration = AnimationSpeed
            };

            childItemsControl.BeginAnimation(HeightProperty, heightDoubleAnimation);

            this.childrenVisible = false;
        }

        #endregion

        #region 배경 설정하기 - SetBackground(background)

        /// <summary>
        /// 배경 설정하기
        /// </summary>
        /// <param name="background">배경</param>
        public void SetBackground(Brush background)
        {
            this.rootGrid.Background = background;
        }

        #endregion
    }
}