﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace TestLibrary
{
    /// <summary>
    /// 사이드 메뉴
    /// </summary>
    public partial class SideMenu
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 테마 속성 - ThemeProperty

        /// <summary>
        /// 테마 속성
        /// </summary>
        public static readonly DependencyProperty ThemeProperty = DependencyProperty.Register
        (
            "Theme",
            typeof(SideMenuTheme),
            typeof(SideMenu)
        );

        #endregion
        #region 상태 속성 - StateProperty

        /// <summary>
        /// 상태 속성
        /// </summary>
        public static readonly DependencyProperty StateProperty = DependencyProperty.Register
        (
            "State",
            typeof(MenuState),
            typeof(SideMenu)
        );

        #endregion
        #region 메뉴 너비 속성 - MenuWidthProperty

        /// <summary>
        /// 메뉴 너비 속성
        /// </summary>
        public static readonly DependencyProperty MenuWidthProperty = DependencyProperty.Register
        (
            "MenuWidth",
            typeof(double),
            typeof(SideMenu)
        );
        
        #endregion
        #region 메뉴 속성 - MenuProperty

        /// <summary>
        /// 메뉴 속성
        /// </summary>
        public static readonly DependencyProperty MenuProperty = DependencyProperty.Register
        (
            "Menu",
            typeof(ScrollViewer),
            typeof(SideMenu)
        );

        #endregion
        #region 그림자 배경 속성 - ShadowBackgroundProperty

        /// <summary>
        /// 그림자 배경 속성
        /// </summary>
        public static readonly DependencyProperty ShadowBackgroundProperty = DependencyProperty.Register
        (
            "ShadowBackground",
            typeof(Brush),
            typeof(SideMenu)
        );

        #endregion
        #region 버튼 배경 속성 - ButtonBackgroundProperty

        /// <summary>
        /// 버튼 배경 속성
        /// </summary>
        public static readonly DependencyProperty ButtonBackgroundProperty = DependencyProperty.Register
        (
            "ButtonBackground",
            typeof(Brush),
            typeof(SideMenu)
        );

        #endregion
        #region 버튼 HOVER 속성 - ButtonHoverProperty

        /// <summary>
        /// 버튼 HOVER 속성
        /// </summary>
        public static readonly DependencyProperty ButtonHoverProperty = DependencyProperty.Register
        (
            "ButtonHover",
            typeof(Brush),
            typeof(SideMenu)
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 표시 여부
        /// </summary>
        private bool isShown;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 테마 - Theme

        /// <summary>
        /// 테마
        /// </summary>
        public SideMenuTheme Theme
        {
            get
            {
                return (SideMenuTheme)GetValue(ThemeProperty);
            }
            set
            {
                if(value == SideMenuTheme.None)
                {
                    return;
                }

                SetValue(ThemeProperty, value);

                SolidColorBrush buttonBackground;
                SolidColorBrush buttonHoverBackground;
                SolidColorBrush background;

                switch(value)
                {
                    case SideMenuTheme.Default :

                        background            = new SolidColorBrush { Color = Color.FromArgb(205, 20, 20, 20) };
                        buttonBackground      = new SolidColorBrush { Color = Color.FromArgb(50 , 30, 30, 30) };
                        buttonHoverBackground = new SolidColorBrush { Color = Color.FromArgb(50 , 70, 70, 70) };

                        break;

                    case SideMenuTheme.Primary :

                        background            = new SolidColorBrush { Color = Color.FromArgb(205, 24, 57 , 85 ) };
                        buttonBackground      = new SolidColorBrush { Color = Color.FromArgb(50 , 35, 85 , 126) };
                        buttonHoverBackground = new SolidColorBrush { Color = Color.FromArgb(50 , 45, 110, 163) };

                        break;

                    case SideMenuTheme.Success :

                        background            = new SolidColorBrush { Color = Color.FromArgb(205, 55, 109, 55) };
                        buttonBackground      = new SolidColorBrush { Color = Color.FromArgb(50 , 65, 129, 65) };
                        buttonHoverBackground = new SolidColorBrush { Color = Color.FromArgb(50 , 87, 172, 87) };

                        break;

                    case SideMenuTheme.Warning :

                        background            = new SolidColorBrush { Color = Color.FromArgb(205, 150, 108, 49) };
                        buttonBackground      = new SolidColorBrush { Color = Color.FromArgb(50 , 179, 129, 58) };
                        buttonHoverBackground = new SolidColorBrush { Color = Color.FromArgb(50 , 216, 155, 70) };

                        break;

                    case SideMenuTheme.Danger :

                        background            = new SolidColorBrush { Color = Color.FromArgb(205, 135, 52, 49) };
                        buttonBackground      = new SolidColorBrush { Color = Color.FromArgb(50 , 179, 69, 65) };
                        buttonHoverBackground = new SolidColorBrush { Color = Color.FromArgb(50 , 238, 92, 86) };

                        break;

                    default :

                        throw new ArgumentOutOfRangeException(nameof(value), value, null);
                }

                ButtonBackground = buttonBackground;
                ButtonHover      = buttonHoverBackground;

                if(Menu != null)
                {
                    Menu.Background = background;

                    StackPanel stackPanel = Menu.Content as StackPanel;

                    foreach(UIElement child in stackPanel.Children)
                    {
                        if(child is MenuButton menuButton)
                        {
                            SetMenuButtonBackground(menuButton, buttonBackground, buttonHoverBackground);
                        }
                    }
                }
            }
        }

        #endregion
        #region 상태 - State

        /// <summary>
        /// 상태
        /// </summary>
        public MenuState State
        {
            get
            {
                return (MenuState)GetValue(StateProperty);
            }
            set
            {
                SetValue(StateProperty, value);

                if(value == MenuState.Visible)
                {
                    Show();
                }
                else
                {
                    Hide();
                }
            }
        }

        #endregion
        #region 메뉴 너비 - MenuWidth

        /// <summary>
        /// 메뉴 너비
        /// </summary>
        public double MenuWidth
        {
            get
            {
                return (double)GetValue(MenuWidthProperty);
            }
            set
            {
                SetValue(MenuWidthProperty, value);
            }
        }

        #endregion
        #region 메뉴 - Menu

        /// <summary>
        /// 메뉴
        /// </summary>
        public ScrollViewer Menu
        {
            get
            {
                return (ScrollViewer)GetValue(MenuProperty);
            }
            set
            {
                SetValue(MenuProperty, value);
            }
        }

        #endregion
        #region 그림자 배경 - ShadowBackground

        /// <summary>
        /// 그림자 배경
        /// </summary>
        public Brush ShadowBackground
        {
            get
            {
                return (Brush)GetValue(ShadowBackgroundProperty);
            }
            set
            {
                SetValue(ShadowBackgroundProperty, value);

                Resources["ShadowSolidColorBrushKey"] = value ?? new SolidColorBrush { Color = Colors.Black, Opacity = .2 };
            }
        }

        #endregion
        #region 버튼 배경 - ButtonBackground

        /// <summary>
        /// 버튼 배경
        /// </summary>
        public Brush ButtonBackground
        {
            get
            {
                return (Brush)GetValue(ButtonBackgroundProperty);
            }
            set
            {
                SetValue(ButtonBackgroundProperty, value);

                Resources["ButtonBackgroundSolidColorBrushKey"] = value;
            }
        }

        #endregion
        #region 버튼 HOVER - ButtonHover

        /// <summary>
        /// 버튼 HOVER
        /// </summary>
        public Brush ButtonHover
        {
            get
            {
                return (Brush)GetValue(ButtonHoverProperty);
            }
            set
            {
                SetValue(ButtonHoverProperty, value);

                Resources["ButtonHoverSolidColorBrushKey"] = value;
            }
        }

        #endregion

        #region 종료 타입 - ClosingType

        /// <summary>
        /// 종료 타입
        /// </summary>
        public ClosingType ClosingType { get; set; }

        #endregion
        #region 표시 여부 - IsShown

        /// <summary>
        /// 표시 여부
        /// </summary>
        public bool IsShown
        {
            get
            {
                return this.isShown;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SideMenu()

        /// <summary>
        /// 생성자
        /// </summary>
        public SideMenu()
        {
            InitializeComponent();

            Theme = SideMenuTheme.Default;

            ClosingType = ClosingType.Auto;

            this.shadowBorder.MouseDown += shadowBorder_MouseDown;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 템플리트 적용시 처리하기 - OnApplyTemplate()

        /// <summary>
        /// 템플리트 적용시 처리하기
        /// </summary>
        public override void OnApplyTemplate()
        {
            Panel.SetZIndex(this, int.MaxValue);

            RenderTransform = new TranslateTransform(-MenuWidth, 0);

            (FindName("menuColumnDefinition") as ColumnDefinition).Width = new GridLength(MenuWidth);

            State            = State;
            Theme            = Theme;
            ShadowBackground = ShadowBackground;
            ButtonBackground = ButtonBackground;
            ButtonHover      = ButtonHover;
        }

        #endregion

        #region 보여주기 - Show()

        /// <summary>
        /// 보여주기
        /// </summary>
        public void Show()
        {
            DoubleAnimation animation = new DoubleAnimation
            {
                From     = -MenuWidth * 0.85,
                To       = 0,
                Duration = TimeSpan.FromMilliseconds(100)
            };

            RenderTransform.BeginAnimation(TranslateTransform.XProperty, animation);

            this.isShown = true;

            (FindName("shadowColumnDefinition") as ColumnDefinition).Width = new GridLength(10000);
        }

        #endregion
        #region 숨기기 - Hide()

        /// <summary>
        /// 숨기기
        /// </summary>
        public void Hide()
        {
            DoubleAnimation animation = new DoubleAnimation
            {
                To       = -MenuWidth,
                Duration = TimeSpan.FromMilliseconds(100)
            };

            RenderTransform.BeginAnimation(TranslateTransform.XProperty, animation);

            this.isShown = false;

            (FindName("shadowColumnDefinition") as ColumnDefinition).Width = new GridLength(0);
        }

        #endregion

        #region 토글하기 - Toggle()

        /// <summary>
        /// 토글하기
        /// </summary>
        public void Toggle()
        {
            if(this.isShown)
            {
                Hide();
            }
            else
            {
                Show();
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그림자 테두리 마우스 DOWN 처리하기 - shadowBorder_MouseDown(sender, e)

        /// <summary>
        /// 그림자 테두리 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void shadowBorder_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if(ClosingType == ClosingType.Auto)
            {
                Hide();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 메뉴 버튼 배경 설정하기 - SetMenuButtonBackground(parentButton, buttonBackground, buttonHoverBackground)

        /// <summary>
        /// 메뉴 버튼 배경 설정하기
        /// </summary>
        /// <param name="parentButton">부모 버튼</param>
        /// <param name="buttonBackground">버튼 배경</param>
        /// <param name="buttonHoverBackground">버튼 HOVER 배경</param>
        private void SetMenuButtonBackground(MenuButton parentButton, Brush buttonBackground, SolidColorBrush buttonHoverBackground)
        {
            parentButton.HoverBackground = buttonHoverBackground;

            parentButton.SetBackground(buttonBackground);

            foreach(MenuButton childButton in parentButton.Children)
            {
                SetMenuButtonBackground(childButton, buttonBackground, buttonHoverBackground);
            }
        }

        #endregion
    }
}